from random import randint, uniform

from . import actors, EAGLE, BIRD, FOOD, FLY
from . import WAIT, HUNT, EXPLORE, RUN, LEWD
from .vec2d import vec2d

class Behavior(object):
    def __init__(self):
        self.image_filename = None
        self.type = None
        self.food = []
        self.hunters = []
        self.counter = 0

    def look_around(self, radius, x_list, y_list):
        neighbors = []
        for actor in actors:
            if (actor.pos - self.actor.pos).get_length() < radius:
                if actor != self.actor:
                    neighbors.append(actor)
        neighbors = sorted(neighbors, key = lambda Actor: int((Actor.pos - self.actor.pos).get_length()))
        return neighbors

    counter = 0

    def change_direction(self, time_passed):
        if self.actor.state == WAIT or self.actor.state == EXPLORE:
            if len(self.actor.target) == 0:
                if self.counter > randint(400, 500):
                    self.actor.direction.rotate(45 * randint(-1, 1))
                    self.counter = 0
                self.counter += time_passed
            else:
                if self.actor.energy > 1.5 * self.actor.dna.energy:
                    part = self.actor.dna.swarm_rejoin
                    rejoin = part
                else:
                    f1, f2, f3 = 0, 0, 0
                    for friend in self.actor.target:
                        r = (friend.pos - self.actor.pos).get_length() / self.actor.dna.sight_radius
                        if r < 0.05:
                            f1 += 0
                            f2 += 20 * r
                            f3 += 1 - 20 * r
                        elif r < 0.1:
                            f1 += 20 * r - 1
                            f2 += 1 - 20 * r
                            f3 += 0
                        else:
                            f1 += 1 - r
                            f2 += 0
                            f3 += 0

                    push = uniform(1, self.actor.dna.swarm_push) * f3
                    pull = uniform(1, self.actor.dna.swarm_pull) * f2
                    rejoin = uniform(1, self.actor.dna.swarm_rejoin) * f1
                    part = max([push, pull, rejoin])

                if part == rejoin:
                    newpos = vec2d(self.actor.pos.x, self.actor.pos.y)
                    for friend in self.actor.target:
                        newpos += friend.pos
                    newpos /= (len(self.actor.target) + 1)
                    diff = newpos - self.actor.pos
                    self.actor.direction = diff.normalized()
                elif part == pull:
                    self.actor.direction = (self.actor.pos - self.actor.target[0].pos).normalized()
                elif part == push:
                    newdirection = vec2d(self.actor.direction.x, self.actor.direction.y)

                    for friend in self.actor.target:
                        newdirection += friend.direction

                    newdirection = newdirection.normalized()
                    self.actor.direction = newdirection

        elif self.actor.state == RUN:
            diff = self.actor.pos - self.actor.target.pos
            self.actor.direction = diff.normalized()
        elif self.actor.state == HUNT or self.actor.state == LEWD:
            prediction = self.actor.target.pos
            self.actor.direction = (prediction - self.actor.pos).normalized()

class Eagle(Behavior):
    def __init__(self):
        Behavior.__init__(self)
        self.image_filename = "artwork/eagle.png"
        self.type = EAGLE
        self.food = [
                    BIRD,
                    FLY,
                    ]
        self.hunters = []

class Bird(Behavior):
    def __init__(self):
        Behavior.__init__(self)
        self.image_filename = "artwork/bird.png"
        self.type = BIRD
        self.food = [
                    FOOD,
                    FLY,
                    ]
        self.hunters = [
                       EAGLE,
                       ]

class Fly(Behavior):
    def __init__(self):
        Behavior.__init__(self)
        self.image_filename = "artwork/fly.png"
        self.type = FLY
        self.food = [
                    FOOD,
                    ]
        self.hunters = [
                        EAGLE,
                        BIRD,
                        ]

class Food(Behavior):
    def __init__(self, image_filename):
        Behavior.__init__(self)
        self.image_filename = image_filename
        self.type = FOOD
        self.food = []
        self.hunters = [
                        BIRD,
                        FLY
                        ]
    def look_around(self, radius, x_list, y_list):
        pass

    def change_direction(self, time_passed):
        pass
