# -*- coding: utf-8 -*-

from os.path import join

import pylab as pl
import numpy as np
import pygame

from . import actors, EAGLE, BIRD, FOOD, FLY

class Plot:

    def __init__(self, directory = None):
        self.directory = directory
        self.X_LENGTH = 1000
        self.y1_length, self.y2_length, self.y3_length = 100, 100, 100
        pl.ion()
        self.ax1 = pl.subplot('221')
        self.creep, = pl.plot([0], [0] , label = "Vogel")
        self.huntline, = pl.plot([0], [0], label = "Adler")
        self.lowline, = pl.plot([0], [0], label = "Fliege")
        self.foodline, = pl.plot([0], [0], label = "Futter")
        pl.legend(bbox_to_anchor = (0., 1.02, 1., .102), loc = 0,
               ncol = 2, mode = "expand", borderaxespad = 0.)
        self.ax2 = pl.subplot('222')
        self.energyline, = pl.plot([0], [0], label = "maximale Energie")
        self.menergyline, = pl.plot([0], [0], label = "durchschnittliche Energie")
        self.henergyline, = pl.plot([0], [0], label = "Adler-maximale Energie")
        self.hmenergyline, = pl.plot([0], [0], label = "Adler-durchschnittliche Energie")
        pl.legend(bbox_to_anchor = (0., 1.02, 1., .102), loc = 0,
               ncol = 2, mode = "expand", borderaxespad = 0.)
        self.ax3 = pl.subplot('223')
        self.cenergyline, = pl.plot([0], [0], label = "Vogel-maximale Energie")
        self.cmenergyline, = pl.plot([0], [0], label = "Vogel-durchschnittliche Energie")
        self.lenergyline, = pl.plot([0], [0], label = "Fliege-maximale Energie")
        self.lmenergyline, = pl.plot([0], [0], label = "Fliege-durchschnittliche Energie")
        self.ax1.set_ylim(0, self.y1_length)
        self.ax2.set_ylim(0, self.y2_length)
        self.ax3.set_ylim(0, self.y3_length)
        self.ax1.set_xlim(0, self.X_LENGTH)
        self.ax2.set_xlim(0, self.X_LENGTH)
        self.ax3.set_xlim(0, self.X_LENGTH)
        pl.legend(bbox_to_anchor = (0., 1.02, 1., .102), loc = 0,
               ncol = 2, mode = "expand", borderaxespad = 0.)
        pl.draw()
        self.count = -1

        self.xlist = []

        self.lcr = []
        self.crp = []
        self.htr = []
        self.food = []
        self.eng = []
        self.meng = []
        self.ceng = []
        self.cmeng = []
        self.heng = []
        self.hmeng = []
        self.leng = []
        self.lmeng = []

        self.data = []

    def update_stats(self):
        self.count += 1

        c, h, f, l = 0, 0, 0, 0
        e, ce, he, le = [], [], [], [],
        for unit in actors:
            if unit.behavior.type == BIRD:
                c += 1
                ce.append(unit.energy)
            if unit.behavior.type == EAGLE:
                h += 1
                he.append(unit.energy)
            if unit.behavior.type == FOOD:
                f += 0.25
            if unit.behavior.type == FLY:
                l += 1
                le.append(unit.energy)
            e.append(unit.energy)
        if c == 0 or h == 0 or l == 0:
            if self.directory != None:
                self.write_to_file(self.directory)
            print "***Eine Spezies ist ausgestorben, halte an!***"
            pl.ioff()
            pl.show()
            pygame.quit()
        else:
            med = np.median(e)
            mx = max(e)
            cmed = np.median(ce)
            cmx = max(ce)
            hmed = np.median(he)
            hmx = max(he)
            lmed = np.median(le)
            lmx = max(le)
            d = [self.count, c, h, f, l, med, mx, cmed, cmx, hmed, hmx, lmed, lmx]
            self.data.append(d)
    
            if self.count <= self.X_LENGTH:
                self.xlist.append(self.count)
                self.crp.append(c)
                self.htr.append(h)
                self.lcr.append(l)
                self.food.append(f)
                self.eng.append(mx)
                self.meng.append(med)
                self.ceng.append(cmx)
                self.cmeng.append(cmed)
                self.heng.append(hmx)
                self.hmeng.append(hmed)
                self.leng.append(lmx)
                self.lmeng.append(lmed)
            else:
                self.crp[self.count % self.X_LENGTH] = c
                self.htr[self.count % self.X_LENGTH] = h
                self.lcr[self.count % self.X_LENGTH] = l
                self.food[self.count % self.X_LENGTH] = f
                self.eng[self.count % self.X_LENGTH] = med
                self.meng[self.count % self.X_LENGTH] = mx
                self.eng[self.count % self.X_LENGTH] = cmed
                self.meng[self.count % self.X_LENGTH] = cmx
                self.eng[self.count % self.X_LENGTH] = hmed
                self.meng[self.count % self.X_LENGTH] = hmx
                self.eng[self.count % self.X_LENGTH] = lmed
                self.meng[self.count % self.X_LENGTH] = lmx
    
    
            if c > self.y1_length:
                self.y1_length = c + 1
                self.ax1.set_ylim(0, self.y1_length)
            if h > self.y1_length:
                self.y1_length = h + 1
                self.ax1.set_ylim(0, self.y1_length)
            if l > self.y1_length:
                self.y1_length = l + 1
                self.ax1.set_ylim(0, self.y1_length)
            if f > self.y1_length:
                self.y1_length = f + 1
                self.ax1.set_ylim(0, self.y1_length)
            if med > self.y2_length:
                self.y2_length = med + 1
                self.ax2.set_ylim(0, self.y2_length)
            if mx > self.y2_length:
                self.y2_length = mx + 1
                self.ax2.set_ylim(0, self.y2_length)
            if cmed > self.y3_length:
                self.y3_length = cmed + 1
                self.ax3.set_ylim(0, self.y3_length)
            if cmx > self.y3_length:
                self.y3_length = cmx + 1
                self.ax3.set_ylim(0, self.y3_length)
            if hmed > self.y2_length:
                self.y2_length = hmed + 1
                self.ax2.set_ylim(0, self.y2_length)
            if hmx > self.y2_length:
                self.y2_length = hmx + 1
                self.ax2.set_ylim(0, self.y2_length)
            if lmed > self.y3_length:
                self.y3_length = lmed + 1
                self.ax3.set_ylim(0, self.y3_length)
            if lmx > self.y3_length:
                self.y3_length = lmx + 1
                self.ax3.set_ylim(0, self.y3_length)
    
            if self.count % 100 == 0:
                self.y1_length = max(
                                     max(self.crp),
                                     max(self.htr),
                                     max(self.lcr),
                                     max(self.food),
                                     ) + 1
                self.y2_length = max(
                                     max(self.ceng),
                                     max(self.cmeng),
                                     max(self.leng),
                                     max(self.lmeng),
                                     ) + 1
                self.y3_length = max(
                                     max(self.heng),
                                     max(self.hmeng),
                                     max(self.eng),
                                     max(self.meng),
                                     ) + 1
                self.ax1.set_ylim(0, self.y1_length)
                self.ax2.set_ylim(0, self.y2_length)
                self.ax3.set_ylim(0, self.y3_length)
    
            self.creep.set_ydata(self.crp)
            self.creep.set_xdata(self.xlist)
            self.huntline.set_ydata(self.htr)
            self.huntline.set_xdata(self.xlist)
            self.lowline.set_ydata(self.lcr)
            self.lowline.set_xdata(self.xlist)
            self.foodline.set_ydata(self.food)
            self.foodline.set_xdata(self.xlist)
            self.energyline.set_ydata(self.eng)
            self.energyline.set_xdata(self.xlist)
            self.menergyline.set_ydata(self.meng)
            self.menergyline.set_xdata(self.xlist)
            self.cenergyline.set_ydata(self.ceng)
            self.cenergyline.set_xdata(self.xlist)
            self.cmenergyline.set_ydata(self.cmeng)
            self.cmenergyline.set_xdata(self.xlist)
            self.henergyline.set_ydata(self.heng)
            self.henergyline.set_xdata(self.xlist)
            self.hmenergyline.set_ydata(self.hmeng)
            self.hmenergyline.set_xdata(self.xlist)
            self.lenergyline.set_ydata(self.leng)
            self.lenergyline.set_xdata(self.xlist)
            self.lmenergyline.set_ydata(self.lmeng)
            self.lmenergyline.set_xdata(self.xlist)
            if self.count % 10 == 0:
                pl.draw()
                print "Step: " + str(self.count)

    def show(self):
        pl.show()

    def write_to_file(self, directory):
        with open(join(directory, "stats" + str(self.count + 1) + ".txt"), "w") as f:
            for d in self.data:
                f.write(" ".join(map(str, d)) + '\n')
